#define MAXLEN (205)
#define MAXPOP (10000)
#define BUFSIZE (100)

/* Base */
#define A (0)
#define C (1)
#define G (2)
#define U (3)

typedef struct __type{
  char seq[MAXLEN];
  char str[MAXLEN];
  double grate; /* growth rate */
  double lambda;
  int dist; /* base pair structure distance */
  int coarse_dist; /* coarse grained distance */
  double energy; /* free energy */
} TYPE;

/* Initilization */
void Init(void);
void InitPopulation(void);

/* molecule annotation */
int AnnotMol(TYPE*);
double CalcLamda(char*);

/* Binary tree (cashing of RNA folding) */
typedef enum {
  STATUS_OK,
  STATUS_MEM_EXHAUSTED,
  STATUS_DUPLICATE_KEY,
  STATUS_KEY_NOT_FOUND
} statusEnum;

typedef struct nodeTag {
  struct nodeTag *left;  /* left child */
  struct nodeTag *right;  /* right child */
  struct nodeTag *parent;  /* parent */
  char key[MAXLEN];  /* key used for searching. sequence of RNA */
  char shape[MAXLEN];  /* shape (folding) of the rna */ 
  double lambda;
  double energy;
  int npop; /* population size */
} nodeType;

int seq_dist(char *, char *);

statusEnum binInsert(char *,nodeType**);
statusEnum binDelete(char *); /* delete a node */
statusEnum binFind(char *, nodeType**);
statusEnum binDeleteTree(nodeType *);
void binRenewTree(void);

/* RNAi2c c2i etc. */
char RNAi2c(int); /* RNA integer to RNA character */
int RNAc2i(char); /* RNA character to RNA integer */

/* Random RNA sequence generator */
void randomRNA(char*,int);

/* Model */
void Reaction(void);
int Replication(char*,char*,double);

/* input */
int Args(int, char **);
void sigcatch(int);
int gotSignal(void);

/* output */
void OutputAll(void);


/* Xmgrace */
void InitXmgraceS(void);
void OutputXmgraceS(void);
void InitXmgraceSeq(void);
void OutputXmgraceSeq(void);
void InitXmgraceE(void);
void OutputXmgraceE(void);
void InitXmgraceC(void);
void OutputXmgraceC(void);
void TermXmgrace(void);
void SaveXmgrace(void);
void RedrawXmgrace(void);

